package cypher;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;


public class LaunchDecryptFile {
    public LaunchDecryptFile() {
        super();
    }
    
    private boolean argsError = false;
    private List listErrorArgs = new ArrayList();
    private HashSet usedFlags = new HashSet();
    private Map options = new HashMap();
    
    private String inputFile;
    private String outputPath;
    private String keyStoreFileName = null;
    private String keyStorePassword = null;
    
    protected DecryptFile decryptFile;
    
    private static Logger LOGGER = Logger.getLogger("cypher.LaunchDecryptFile");
    
    /**
     * Executable method.
     * @param args: Args must be a combination of parameter specification (flag, -something) and value (without -) see 
     * initUsedArgs for a list of parameters specification (-k keystoreFile -p keystorePassword -f FileToUnEncrypt -o OutputPath)
     */
    public static void main(String[] args) {
        LaunchDecryptFile decrypt = new LaunchDecryptFile();
        if (decrypt.loadArgs(args)) {
            decrypt.launch();
        }
    }
    
    /**
     * Create a DecryptFile object and launch it (via constructor)
     */
    protected void launch() {
        try {
            decryptFile = new DecryptFile(this.inputFile, this.outputPath, this.keyStoreFileName, this.keyStorePassword);
        } catch (Exception e) {
            LOGGER.warning(e.getMessage());
        }
    }
    
    /**
     * Load and initilize arguments.
     * @param args: Arguments, same as in main.
     * @return: true if everything is ok. false if there is any problem. (note, true/false as boolean primitive, not Boolean object).
     */
    protected boolean loadArgs(String[] args) {
        this.initUsedArgs();
        boolean resFlag = false;
        parse(args);
        String resStr = checkArgs();
        if (resStr.length() == 0) {
            resFlag = true;
        } else {
            resFlag = false;
            LOGGER.warning("Syntax error/s:\n" +
                    resStr);
        }
        return resFlag;
    }
    
    /**
     * Initializes usedFlag with a list of allowed flags (parameter specifications)
     */
    private void initUsedArgs() {
        usedFlags.add("-f"); //File to validate/extract signature.
        usedFlags.add("-o"); //Path to generate zip etc . . .
        usedFlags.add("-k"); //Java KeyStore file
        usedFlags.add("-p"); //Java KeyStore password
    }
    
    /**
     *Do whatever checking on the arguments
     * @return: String with the error/s (if any)
     */
    private String checkArgs() {
        String strRes = "";
        String param, message;
        if (options == null || options.isEmpty()) {
            message = "Use with parameters: " + "-k keystoreFile " + "-p keystorePassword " + "-f FileToUnEncrypt " + "-o OutputPath";
            strRes = strRes + message;
        } else {
            param = "-k";
            message = "Missing parameter -k. Use: -k keystoreFile\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                keyStoreFileName = ((String)options.get(param)).trim();
            }
            param = "-p";
            message = "Missing parameter -p. Use: -p keystorePassword\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                keyStorePassword = ((String)options.get(param)).trim();
            }
            param = "-f";
            message = "Missing parameter -f. Use: -f FileToUnEncrypt\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                inputFile = ((String)options.get(param)).trim();
            }
            param = "-o";
            message = "Missing parameter -o. Use: -o OutputPath\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                outputPath = ((String)options.get(param)).trim();
            }
        }
        return strRes;
    }
    
    /**
     * Parse parameters from arguments to its corresponding place (inside options)
     * @param args: Same as in the main method
     * @return: true if everything is ok. false otherwise.
     */
    private boolean parse(String[] args) {
        for (int n = 0; n < args.length; ++n) {
            if (args[n].charAt(0) == '-') {
                String name = args[n];
                String value = null;

                if (usedFlags.contains(args[n]) && n < args.length - 1) {
                    value = args[++n];
                    options.put(name, value);
                }
            }

            else {
                listErrorArgs.add(args[n]);
                argsError = true;
            }
        }
        return argsError;
    }

    /**
     * Utility method to get de decrypted file name.
     * @return String with the decrypted file name.
     */
    public String getDecryptFileName(){
        String res="";
        if (decryptFile!=null){
            res=decryptFile.getEmbeddedFileName();
        }
        return res;
    }
}